	--[[ Depth-First Search Maze
	 http://www.mazeworks.com/mazegen/mazetut/index.htm
	 Sodium 04.01.2013
	 Controls:
	 'r' reload;	'x' start;	'return' finish;	'p' pause;	'lmb' block;	'rmb' bomb;--]]

function love.load()
	grid={}
	tile=18;		--smallest numbers, bigger mazes and crap fps.
	bmb={};
	SW,SH=love.graphics.getWidth(),love.graphics.getHeight();
	audio={};
	audio.bmb=love.audio.newSource("bmb.wav","static");
	audio.clk=love.audio.newSource("clk.wav","static");
	wmax=math.floor(SW/tile);
	hmax=math.floor(SH/tile);
	for i=1,wmax do
		grid[i]={}; bmb[i]={};
		for j=1,hmax do
			grid[i][j]={x=i-1,y=j-1,v=false,ck=false,w={0,0,0,0}}; bmb[i][j]={x=i-1,y=j-1};
		end
	end
	ncells = wmax*hmax;
	cur = {math.random(wmax),math.random(hmax)};		--current cell
	frst = cur
	grid[cur[1]][cur[2]].v=true;
	old = 1;											--counter
	undo = {};											--history
	undo[1]=cur;
	mode=0;
	mx,my=0,0;
	stp=0;
	Dt=0;
	fire=Fire(60,30)
end

function love.update(dt)
	Dt=dt*30;
	if mode>1 then
		while old < ncells do
			cur = FindNeighbors(cur) or undo[#undo];
		end
	elseif mode==1 and old < ncells then
		stp = stp + dt*2;
		if stp>.1 then
			cur = FindNeighbors(cur) or undo[#undo]; stp=0;
		end
		if bmb[cur[1]][cur[2]].e then bmb[cur[1]][cur[2]].e.READY=true; love.audio.play(audio.bmb); love.audio.rewind(audio.bmb); end 
	end
	mx,my=love.mouse.getPosition();
	love.graphics.setCaption("A MAZE - FPS: "..tostring(love.timer.getFPS( )));
end

function love.draw()
	
	love.graphics.setColor(100,222,100,60)
	love.graphics.rectangle("fill",1,1,wmax*tile,hmax*tile) --Green BG
	for i=1,wmax do
		for j=1,hmax do
			love.graphics.setColor(0,0,0)
			local x,y=(i-1)*tile,(j-1)*tile;
			if not grid[i][j].v then love.graphics.rectangle("fill",(x)+tile/4,(y)+tile/4,tile/2,tile/2) end --black square
			
			-- WALLS 
			if grid[i][j].w[3]==0 then love.graphics.rectangle("fill",x+tile-1,y,2,tile) end
			if grid[i][j].w[4]==0 then love.graphics.rectangle("fill",x,y+tile-1,tile,2) end
			--
			-- show bombs and blocks
			if bmb[i][j].e then 
				bmb[i][j].e:draw()
				if not grid[i][j].v then
				love.graphics.setColor(200,20,20)
				love.graphics.rectangle("fill",(x)+tile/4,(y)+tile/4,tile/2,tile/2) 
				end
				if #bmb[i][j].e<=1 then bmb[i][j].e=false end
				love.graphics.setColor(0,0,0)
			end 
			if grid[i][j].ck then
				love.graphics.setColor(40,105,170)
				love.graphics.rectangle("fill",(x)+tile/4,(y)+tile/4,tile/2,tile/2)
			end
		end
	end
	-- first yellow , current red
	love.graphics.setColor(255,255,0)
	love.graphics.rectangle("fill",(grid[frst[1]][frst[2]].x*tile)+tile/4,(grid[frst[1]][frst[2]].y*tile)+tile/4,tile/2,tile/2)
	love.graphics.setColor(255,0,0)
	love.graphics.rectangle("fill",(grid[cur[1]][cur[2]].x*tile)+tile/4,(grid[cur[1]][cur[2]].y*tile)+tile/4,tile/2,tile/2)
	
	fire:draw((grid[cur[1]][cur[2]].x*tile)+tile/4,(grid[cur[1]][cur[2]].y*tile)+tile/4)		--FIRE!!
	
	love.graphics.setColor(255,255,255)
	love.graphics.rectangle("line",1,1,wmax*tile,hmax*tile)		--border
	if love.keyboard.isDown(" ") then							--info
		love.graphics.setColor(0,0,0,100)
		love.graphics.rectangle("fill",0,0,SW,SH)
		love.graphics.setColor(255,255,255)
		local txt="Controls:\n'r' reload;\t'x' start;\t'return' finish;\t'p' pause;\t'lmb' block;\t'rmb' bomb\n'Esc' quit;"
		love.graphics.printf(txt,0,0,SW,"center")
	end
	if mode==0 then   --if paused
		love.graphics.setColor(0,0,0)
		love.graphics.rectangle("fill",335,SH-42,130,34)
		love.graphics.setColor(255,255,255)
		love.graphics.printf("PRESS 'X' TO START.\nhold space for more",0,SH-40,SW,"center")
	end
	love.graphics.setColor(0,0,0)
	love.graphics.rectangle("fill",750,0,50,14)
	love.graphics.setColor(255,255,255)
	love.graphics.printf((math.ceil(10*old/ncells*100)/10).."%",0,0,SW-10,"right")		--percent
	
end

function love.keypressed(k)
	if k=="escape" then love.event.quit() end
	if k=="r" then love.load() end
	if k=="x" then mode=1;
	elseif k=="return" then mode=2;
	elseif k=="p" then mode=mode==1 and 0 or 1; end
end

function love.mousepressed(mx,my,mb)
	if mb=="l" then
		for i=0,wmax do
			for j=0,hmax do
				local x,y=-tile+(i*tile),-tile+(j*tile);
				if coll(x,y,tile,tile) and not grid[i][j].v and not bmb[i][j].e then
				grid[i][j].ck=true; grid[i][j].v=true;
				love.audio.play(audio.clk); love.audio.rewind(audio.clk); end
			end
		end
	end
	if mb=="r" then
		for i=0,wmax do
			for j=0,hmax do
				local x,y=-tile+(i*tile),-tile+(j*tile);
				if coll(x,y,tile,tile) and not grid[i][j].v and not bmb[i][j].e then
				bmb[i][j].e=Particle(x,y,60,30);
				love.audio.play(audio.clk); love.audio.rewind(audio.clk); end
			end
		end
	end
end

function FindNeighbors(c)
	local cell={};
	if c[1]>1 and not grid[c[1]-1][c[2]].v and not grid[c[1]-1][c[2]].ck then		--check left
		table.insert(cell,{c[1]-1,c[2],w=1})
	end
	if c[2]>1 and not grid[c[1]][c[2]-1].v and not grid[c[1]][c[2]-1].ck then		--check up
		table.insert(cell,{c[1],c[2]-1,w=2})
	end
	if c[1]<wmax and not grid[c[1]+1][c[2]].v and not grid[c[1]+1][c[2]].ck then	--check right
		table.insert(cell,{c[1]+1,c[2],w=3})
	end
	if c[2]<hmax and not grid[c[1]][c[2]+1].v and not grid[c[1]][c[2]+1].ck then	--check down
		table.insert(cell,{c[1],c[2]+1,w=4})
	end
	
	if #cell>0 then
		table.insert(undo,c);
		old=old+1;
		local n=cell[math.random(#cell)];		--choose random side
		grid[c[1]][c[2]].w[n.w]=1;				--Walls related
		grid[n[1]][n[2]].w[n.w>2 and n.w-2 or n.w+2]=1;	--Walls related
		grid[n[1]][n[2]].v=true;				--visited
		return n;
	else
		if #undo>1 then table.remove(undo) else old = ncells; end
	end
end

function coll(x2,y2,w2,h2)
	return mx<=(x2+w2) and mx>=x2 and my<=(y2+h2) and my>=y2;
end;

function Fire(max,life)
	local a={}
	for i=1,max do
		table.insert(a,{x=0,y=0,h=math.random(20,50),l=0});
	end
	function a:draw(X,Y)
		for i,k in ipairs(self) do
			if not pause then
				k.l=k.l-math.random(5);
				if k.l<=0 then
					k.x=X;
					k.y=Y;
					k.l=life;
				else
					k.y=k.y-math.random(6)*Dt;
					k.x=k.x+math.random(-2,2)*Dt;
				end
			end

			love.graphics.setColor(255,0,0,200);
			love.graphics.point(k.x+math.random(),k.y+math.random())	--sparks
			love.graphics.setColor(255,255,0,160);
			love.graphics.circle("fill",k.x,k.y,k.l/5,6)
		end
	end
	return a;
end;

function Particle(X, Y, Max, Life)
	local this={}
	LIFE  = Life
	SPEED = 25;	-- max speed
	this.READY = false;
	for i=1,Max do
		local u={}
		u.x = X;
		u.y = Y;
		u.size = math.random(1, 4);
		u.l = LIFE;
		local ang= math.rad(math.random(360))
		u.xv = math.cos(math.random(SPEED))*ang;
		u.yv = math.cos(math.random(SPEED))*ang;
		u.a = 255;
		-- smoothing out the diagonal speed
		if (u.xv * u.xv + u.yv * u.yv) > (SPEED * SPEED) then
			u.xv = u.xv * 0.7;
			u.yv = u.yv * 0.7;
		end
		table.insert(this,u)
	end

	function this:draw()
		if this.READY then
			for i,k in ipairs(this) do
				if k.l > 0 then
					k.x = k.x + k.xv*Dt;
					k.y = k.y + k.yv*Dt;
					k.a = k.a>0 and k.a-5 or 0;
					k.l = k.l-1;
				else
					table.remove(this)
				end
				love.graphics.setColor(250,20,20,k.a);
				love.graphics.rectangle("fill",k.x, k.y, k.size, k.size);
			end
		end
	end
	return this;
end
